'''This example demonstrates the use of Convolution1D for text classification.

Run on GPU: THEANO_FLAGS=mode=FAST_RUN,device=gpu,floatX=float32 python imdb_cnn.py

Get to 0.835 test accuracy after 2 epochs. 100s/epoch on K520 GPU.
'''

from __future__ import print_function
import numpy as np
import random
from ChallengeSubmitLib import FIRFilters

np.random.seed(1337)  # for reproducibility
import keras
from keras.models import model_from_json
import sys
import wfdb
import scipy.io as sio
# set parameters:
random.seed(3997)
SaveName = "rawFilterArrayModify8s"

fs = 300
cutLength = 8 * fs
filename = sys.argv[1]
input_num = 41
inputTickdim = 3 * cutLength
inputFeatureDim = 4


######################################################
def filterFun(sig, fs=300, lowPass=1, highPass=150):
    f = FIRFilters()
    filtecg = f.highpass(sig, fs, lowPass)
    filtecg = f.lowpass(filtecg, fs, highPass)
    return filtecg
def getStartNextMaxValuePositon( signal, fs, startPosition):
    absolute_position = startPosition + np.argmax(signal[startPosition:startPosition + fs])
    # relative_position = np.argmax(signal[startPosition:startPosition + fs])
    return absolute_position

def getEndLastMaxValuePositon( signal, fs, endPosition):
    absolute_position = endPosition + np.argmax(signal[endPosition:endPosition + fs])
    # relative_position = np.argmax(signal[endPosition:endPosition + fs])
    return absolute_position

def GetRawFilterSegmentArray_Modify(sig, fs=300, segLength=3 * 300, segNum=3):
        randomSegment_Array = []
        segmentAll = []
        sig=sig[fs:]#first second is not realiable
        if (len(sig) > 25 * fs):
            allLength = segLength * segNum
            sig_cutLastsection = len(sig) - allLength
            randomPosition = random.randint(0, sig_cutLastsection - 1)
            segmentAll = sig[randomPosition:randomPosition + allLength]
        else:
            sig_cutLastsection = len(sig) - segLength - fs  # avoid segment exceed sig max length
            for k in range(segNum):
                randomPosition = random.randint(0, sig_cutLastsection - 1)
                start_pos = randomPosition
                end_pos = start_pos + segLength + fs
                if k == 0:
                    start_pos = start_pos  # first start position don't need change
                else:
                    start_pos = getStartNextMaxValuePositon(sig, fs, start_pos)
                if k == segNum - 1:
                    end_pos = end_pos  # last end position extend 1 second ,For simplifying caculation
                else:
                    end_pos = getEndLastMaxValuePositon(sig, fs, end_pos)
                sig_segment = sig[start_pos:end_pos]
                sig_segment = normalize_DivMax(sig_segment)
                segmentAll[len(segmentAll):len(sig_segment)] = sig_segment
            if (len(segmentAll) > segLength * segNum):
                segmentAll = segmentAll[:segLength * segNum]
            segmentAll = np.asarray(segmentAll)
        # seconds normalize 
        segmentAll = normalize(segmentAll)
        sig_filter1 = filterFun(segmentAll, fs, 1, 70)
        sig_filter2 = filterFun(segmentAll, fs, 5, 90)
        sig_filter3 = filterFun(segmentAll, fs, 20, 120)
        randomSegment_Array.append(segmentAll)
        randomSegment_Array.append(sig_filter1)
        randomSegment_Array.append(sig_filter2)
        randomSegment_Array.append(sig_filter3)
        randomSegment_Array = np.asarray(randomSegment_Array)  # get 4*6s array
        # print(randomSegment_Array.shape)
        return randomSegment_Array


def write_answer(filename, result, resultfile="answers.txt"):
    fo = open(resultfile, 'a')
    fo.write(str(filename) + "," + str(result) + "\n")
    fo.close()


def oneHotToNumber(y_vector):
    predict = []
    for itr in y_vector:
        predict.append(np.argmax(itr))
    return np.asarray(predict)

def normalize(signal):
	sig=signal.copy()
	mean = np.mean(sig)
	std = np.std(sig)
	filtecg = (sig - mean) / std
	return filtecg
def normalize_DivMax(signal, bits=None):
	'''
	normalize to be in a given range. The default is to normalize the maximum
	amplitude to be one. An optional argument allows to normalize the signal
	to be within the range of a given signed integer representation of bits.
	'''

	s = signal.copy()

	s /= np.abs(s).max()

	# if one wants to scale for bits allocated
	if bits is not None:
		s *= 2 ** (bits - 1)
		s = clip(signal, 2 ** (bits - 1) - 1, -2 ** (bits - 1))
	return s
def vote(y_predict):
    voteArray=[0,0,0,0]
    for i in y_predict:
        voteArray[i]=voteArray[i]+1
    print(voteArray)
    voteArray=np.asarray(voteArray)
    if(np.argmax(voteArray)==0):
        voteArray[2]=(sum(voteArray)/12)*voteArray[2]#other multi 1.5,multi voteNum/10	
    return np.argmax(voteArray)
################################################################
# paper data
print('ReadData:', str(filename))
model_input = []
sig, fields = wfdb.srdsamp(str(filename))
#sig = sio.loadmat(filename+'.mat')
#sig = sig['val']
#sig = sig.astype(np.float64)
sig = sig.reshape([1, -1]).flatten()  # 9000*1 tranform to 1*9000

print(len(sig),cutLength+fs+1)
if(len(sig)>cutLength+2*fs+1):
	#sig = normalize(sig)
	print(sig.shape)
	for i in range(input_num):
		oneInput = GetRawFilterSegmentArray_Modify(sig, fs, cutLength, segNum=3)
		model_input.append(oneInput)
	model_input = np.asarray(model_input)
	model_input = np.swapaxes(model_input, 1, 2)
	model_input = np.reshape(model_input, (-1, inputTickdim, inputFeatureDim))
	print(model_input.shape)
	# papare model
	print('Build model...')
	loss = 'categorical_crossentropy'
	optimizer = 'Adam'
	model = model_from_json(open(SaveName + '.json').read())
	model.load_weights(SaveName + '.h5')
	model.compile(loss=loss, optimizer=optimizer, metrics=['accuracy'])
	y_pred = model.predict(model_input, batch_size=input_num)
	y_predict = oneHotToNumber(y_pred)
	#print(y_predict.shape)
	average_prediction = vote(y_predict)
else:
	average_prediction=0
# for i in y_predict:
#     print(i)

if average_prediction == 0:
    label = 'N'
elif average_prediction == 1:
    label = 'A'
elif average_prediction == 2:
    label = 'O'
else:
    label = '~'
filename = filename.split('/')[-1]
write_answer(filename=filename, result=label)
